<?php
/**
 * Функции для непосредственного импорта товаров и экспорта продаж
 * 
 * Преобразуют данные из формата WooCommerce в формат Subtotal
 * 
 * @package Subtotal plugin
 * @author Царёв Владимир <sau412@arikado.xyz>
 */

/**
 * Импортирует продукт из Subtotal в WooCommerce
 * 
 * @param array $product Данные продукта в формате Subtotal
 * @param bool $auto Признак импорта по расписанию
 */
function st_insertOrUpdateProduct($good, $auto) {
	global $modx;
	static $subtotal_imported_ids = [];

	$good_id = $good['id'];

	st_log_info(["function" => "st_insertOrUpdateProduct", "good_info" => $good, "auto" => $auto]);

	// Проверяем что у товара хотя бы один тег из списка тегов, указанных пользователем
	$subtotal_import_tags_list = st_get_import_tags_list();
	$subtotal_import_tags_list_array = json_decode($subtotal_import_tags_list);
	if(st_checkProductInTags($good, $subtotal_import_tags_list_array) == false) {
		st_log("Product $good_id has no tags: ".implode(", ", $subtotal_import_tags_list_array));
		return;
	}

	// Опции импорта
	$update_stocks = st_get_update_stocks();
	if ($auto) {
		$import_name = st_get_import_auto_name();
		$import_photos = st_get_import_auto_photos();
		$import_description = st_get_import_auto_description();
 		$import_tags = st_get_import_auto_tags();
	} else {
		$import_name = 1;
		$import_photos = st_get_import_photos();
		$import_description = st_get_import_description();
		$import_tags = st_get_import_tags();
	}

	// Группа modx в которую будут привязываться товары
	$modx_group = st_get_import_to_group();

	// Шаблон modx который будет применяться ко всем товарам
	$modx_template = st_get_import_to_template();

	// Ищем этот продукт среди уже загруженных по метаданным
	$modx_id = st_get_modx_id_by_subtotal_id($good_id);
	if($modx_id) {
		st_log("st_get_product_by_id($modx_id)");
		$product = st_get_product_by_id($modx_id);
	}

	// Ищем продукт по названию (может быть актуально при смене бизнесов, когда меняются идентификаторы, но не имена)
	if(!$product) {
		st_log("Product not found by subtotal_id, try to find with the name");
		$product = st_get_product_by_name($good['name']);
	}

	// Проверяем что этот продукт уже есть, и если установлено "только новые продукты" - ничего не делаем
	if ($product && isset($_POST) && $_POST['only_new_products'] == 'true') {
		st_log("Product exists, only new products set, skipping");
		return $product;
	}

	// details - либо сам продукт, либо продукт, вариацией которого является текущий продукт
	$details = st_getProductDetailed($good_id);
	st_log_debug(["function_call" => "st_getProductDetailed", "result" => $details]);

	// id базового продукта будет отличаться для вариаций
	$base_product_id = $details['good']['id'];
	if(!$base_product_id) {
		st_log_warning(["no_product_details"]);
		return;
	}

	// Выяснение цены на продукт по выбранному прайсу
	$price_id = st_get_import_price();
	//$price_value = st_getProductPriceValue($details['good'], $price_id);
	$price_value = st_getProductPriceValue($good, $details['good'], $price_id);

	// Если продукта нет - создаём его
	if(!$product) {
		$product = $modx->newObject('msProduct');

		// В состоянии "опубликован"
		$product->set("published", true);
		st_log("Create new product");

		// Алиас устанавливается в момент создания продукта и не меняется после
		$product->set("alias", $good['name']);
	}

	if($import_name) {
		$product->set("pagetitle", $good['name']);
	}

	if($import_tags) {
		$product->set("tags", $good['tags']);
	}
	if($import_description) {
		if(preg_match_all('/<p>([^<:]+): ?<span>([^<]+)<\\/span><\\/p>/',
				$details['good']['description'], $matches)) {
			foreach($matches[1] as $key => $field_name) {
				$field_value = $matches[2][$key];
				st_log_debug(["name" => $field_name, "value" => $field_value]);
				if(in_array($field_name, ["description"])) {
					$product->set("description", $field_value);
				}
				else {
					st_set_tv_value($product, $field_name, $field_value);
				}
			}
		}
		else {
			st_log_debug(["message" => "No format for description", "details" => $details['good']['description']]);
			$product->set("description", $details['good']['description']);
		}
	}

	$product->set("article", $good['article']);
	$product->set("price", $price_value);
	$product->set("parent", $modx_group);
	$product->set("template", $modx_template);
//	$product->set("subtotal_id", $good_id);

	// TVs
	st_set_tv_value($product, "barcode", $good['barcode']);
	st_set_tv_value($product, "subtotal_id", $good['id']);
	st_set_tv_value($product, "unit", $good['unit_name']);
	st_set_tv_value($product, "stock", $good['avial']);
	st_set_tv_value($product, "weight", $good['weight']);
	st_set_tv_value($product, "country", $good['country_name']);

	$product->save();

	// Импорт главного фото и галереи (если включен)
	if(!empty($import_photos)) {
		$image_urls_array = [];
		if($good['default_images']['full']) {
			$image_urls_array[] = SUBTOTAL_API.$good['default_images']['full'];
		}
		$photo_data = st_getProductPhoto($product, $image_urls_array);
/*		$images_array = [];
		if(isset($details['good']['image1']['url'])) array_push($images_array, $details['good']['image1']['url']);
		if(isset($details['good']['image2']['url'])) array_push($images_array, $details['good']['image2']['url']);
		if(isset($details['good']['image3']['url'])) array_push($images_array, $details['good']['image3']['url']);
		if(isset($details['good']['image4']['url'])) array_push($images_array, $details['good']['image4']['url']);

		$photo_data = st_getProductPhoto($details['good'], $images_array);
		if($photo_data['main_image']) {
			$wc_product->set_image_id($photo_data['main_image']);
		}
		if(count($photo_data['gallery'])>0) {
			$wc_product->set_gallery_image_ids($photo_data['gallery']);
		}*/
		$product->save();
	}

	st_log("Good $good_id saved");
	// Метаданные чтобы потом найти этот продукт
	//update_post_meta($wc_product_id, '_subtotal_data', $subtotal_data);
	//update_post_meta($wc_product_id, '_subtotal_id', $base_product_id);
	//$subtotal_imported_ids[] = $base_product_id;
	//update_post_meta($wc_product_id, '_subtotal_updated', time());
	st_set_ids_link($product->get("id"), $good_id);
	st_reset_cache();
	return $product;
}

/**
 * Экспортирует продажу из miniShop2 в Subtotal
 * 
 * @param object $order Продажа
 * @param array $data Данные продажи
 */
function subtotal_export($order) {
	global $modx;
	st_log("subtotal_export started");
//var_dump($order);
	$access = st_get_session();
	if (!$access) return;

	$pos_id = st_get_stock_store();
	if (!$pos_id) return;

	$price = st_get_import_price();
	if (!$price) return;

        $poses = st_getPoses();
        $store_id = '';
        foreach($poses['poses'] as $pos_data) {
                if($pos_id == $pos_data['id']) {
                        $store_id = $pos_data['store_id'];
                        break;
                }
        }
        if(!$store_id) return;

	// Номер заказа (можно ещё id)
	$order_id = $order->get("id");
	$order_num = $order->get("num");
	st_log("Order id: $order_id");

	// Стоимость заказа
	$total_price = $order->get("cost");
	//st_log("Total price: $total_price");

	// Найти или создать пользователя, сначала по e-mail
	// E-mail есть только в профиле пользователя
	$user_id = $order->get("user_id");
	$userProfile = $modx->getObject('modUserProfile', $user_id);

	// Адрес есть в заказе
	$address_id = $order->get("address");
	$address = $modx->getObject('msOrderAddress', $address_id);
//var_dump($userProfile);
	$data = [
		"receiver" => $address->get("receiver"),
		"email" => $userProfile->get("email"),
		"phone" => $address->get("phone"),
		"index" => $address->get("index"),
		"country" => $address->get("country"),
		"region" => $address->get("region"),
		"city" => $address->get("city"),
		"street" => $address->get("street"),
		"building" => $address->get("building"),
		"room" => $address->get("room"),
		"comment" => $address->get("comment"),
	];
st_log(["user_data" => $data]);
	$partner_found = false;
	$partner_email = $data['email'];
	$partner_phone = $data['phone'];
	$address = "Индекс '".$data['index']."'";
	$address .= ", регион '".$data['region']."'";
	$address .= ", город '".$data['city']."'";
	$address .= ", улица '".$data['street']."'";
	$address .= ", дом '".$data['building']."'";
	$address .= ", комната '".$data['room']."'";

	//st_log("Partner: email '$partner_email' phone '$partner_phone' address '$address'");

	if($partner_email != '') {
		$partners_exists_array = st_filterPartners($partner_email);
		foreach($partners_exists_array['partners'] as $partner_data) {
			if($partner_data['email'] == $partner_email) {
				$partner_found = true;
				$partner_id = $partner_data['id'];
				break;
			}
		}
	}

	// Поиск по телефону немного отличается от поиска по email
	if($partner_found == false && $partner_phone != '') {
		$partners_exists_array = st_filterPartners($partner_phone);
		foreach($partners_exists_array['partners'] as $partner_data) {
			$partner_found = true;
			$partner_id = $partner_data['id'];
			break;
		}
	}

	// Если клиента нет - создаём нового
	if($partner_found == false) {
		// Только если заполено хоть какое-то поле
		if($data['receiver'] || $partner_email || $partner_phone) {
			//st_log("Create new partner");
			$partner_data = [
				"contacts" => [[
					"fname" => $data['receiver'],
					"lname" => "",
					"mname" => "",
					"email" => $partner_email,
					"phone" => $partner_phone,
					"addr_fact" => $address,
				]],
				"partner" => [
					"legal_type_id" => 3,
					"partner_type_id" => 5,
					"email" => $partner_email,
					"phone" => $partner_phone,
					"description" => "Создан автоматически к заказу #".$order_id,
				]
			];
//st_log($partner_data);
			$new_partner_data = st_createPartner($partner_data);
//st_log($new_partner_data);
			$partner_id = $new_partner_data['id'];
		}
		else {
			$partner_id = null;
		}
	}
//return;
	// Комментарий к заказу
	$order_date = $order->get('createdon');
	$comment =  $order_date." создан заказ #".$order_id;
	if (!empty($data['comment'])) $comment.="\n\rКомментарий покупателя: ".$data['comment'];
	if (!empty($address)) $comment.="\n\rАдрес: $address";

	if(empty($sale_uid)) {
		$sale_uid = $store . "-" . microtime();
	}

	$sale = [
		'goods' => [],
		'outcome' => [
			'pos_id' => $pos_id,
			'store_id' => $store_id,
			'description' => $comment,
			'partner_id' => $partner_id,
		],
		'price' => $total_price,
		'uid' => $sale_uid,
	];

	// Заполнить продукты
	$goods = $modx->getIterator('msOrderProduct', array('order_id' => $order_id));

	// Получаем id продажи в subtotal
	$exists_sale_id = get_subtotal_sale_id_by_modx_sale_id($order_id);

	foreach ($goods as $item) {
		$modx_good_id = $item->get("product_id");
		//var_dump("modx_good_id $modx_good_id");
		$subtotal_good_id  = st_get_subtotal_id_by_modx_id($modx_good_id);
if(!$subtotal_good_id) {
st_log("st_get_subtotal_id_by_modx_id($modx_good_id) = $subtotal_good_id");
die();
}
		$good = [
			"id" => intval($subtotal_good_id),
		];

		$sale['goods'][] = [
			"id" => null,
			"cnt" => $item->get('count'),
			"good" => $good,
			"price" => $item->get('price'),
		];

		// Уменьшаем количество продукта в TV-полях
		// Происходит только при создании продажи в subtotal
		if(!$exists_sale_id) {
			$product = st_get_product_by_id($modx_good_id);
			$stock = st_get_tv_value($product, "stock");
			$count_sold = $item->get('count');
			st_set_tv_value($product, "stock", $stock - $count_sold);
		}
	}

	st_log(["Sale" => $sale]);
	//var_dump($sale);
	//return;
// $modx->getObject('modUserProfile', $addr_user_id);
 	$result = st_exportSale($sale, $exists_sale_id);

	st_log(["function_called" => "st_exportSale", "result" => $result]);
//	st_log(["Result" => $result]);

	if($result['id']) {
		$sale_id = $result['id'];
		set_subtotal_sale_id_by_modx_sale_id($order_id, $sale_id);

		$sale_data = st_getSale($sale_id);

		// Отмечаем как отгруженный
		$sale_data['outcome']['is_carry_out'] = true;
		$result = st_exportSale($sale_data, $exists_sale_id);
		st_log(["sale_data" => $sale_data, "exists_sale_id" => $exists_sale_id, "result" => $result]);

		$order_status = $order->get("status");
//var_dump("order status $order_status");
		// 2 - заказ оплачен, ничего кроме идентификатора тут использовать не выходит
		if($order_status == 2) {
			// Если оплата наличными курьеру - это наличные
			$pay_type = 1; // Наличные
			$payData = [
				"description" => "Заказ #".$order_data['id']." помечен как оплаченный (".$order_data['status'].") способом ".$order_data['payment_method_title'],
				"no_shift" => 0,
				"pay_date" => date("Y-m-d")."T".date("H:i:s.v")."Z", // Пример 2020-05-13T13:50:51.598Z
				"pay_sum" => $total_price,
				"pay_type" => 1,
				"pos" => $pos_id,
			];
//var_dump($payData);
			$result = st_payOrder($sale_id, $payData);
//var_dump($result);
		}
	}
}

/**
 * Устанавливает фото продукта
 * 
 * @param array $product Данные о продукте (чтобы подставить название)
 * @param array $images_array Массив URL с фотографиями
 */
function st_getProductPhoto($product, $images_array) {
	$result = [
		"main_image" => null,
		"gallery" => [],
	];
	$main_image_set = false;

	if(count($images_array) > 0) {
		foreach($images_array as $image_url) {
			$id = st_getImagePostId($product, $image_url);
			if(!$main_image_set) {
				$result['main_image'] = $id;
				$main_image_set = true;
			}
			else {
				$result['gallery'][] = $id;
			}
		}
	}
	return $result;
}

/**
 * Загружает картинку и возвращает её id
 * 
 * @param string $product_name Название продукта будет установлено как название картинки
 * @param string $image_url Ссылка на картинку
 * 
 * @return int Идентификатор картинки в WordPress
 */
function st_getImagePostId($product, $image_url) {
	global $modx;
	$id = null;
	// Delete all images
	$files_array = $product->getMany('Files', array('parent' => 0));
	foreach($files_array as $file) {
		$file->remove();
	}
	// Load new images
	if($image_url != '') {
		// Ссылка может меняться, а вот хэш картинки - нет
		$image_contents = st_getImage($image_url);
		// Часть после последнего / - имя файла
		preg_match('/\\/([^\\/]+)$/', $image_url, $matches);
		$image_name = $matches[1];
		$hash = hash("sha256", $image_contents);
		$image_file = MODX_ASSETS_PATH."images/".$image_name;
//		st_log($image_url);
//		st_log(MODX_ASSETS_PATH."images/".$image_name);
		file_put_contents($image_file, $image_contents);
		$file = [
			"id" => $product->get("id"),
			"name" => $image_name,
			"file" => $image_file,
		];
		$processorPath = MODX_CORE_PATH . 'components/minishop2/convert/ms1/processors/';
		require $processorPath . '/config.inc.php';
//echo "\nBefore run processor\n";
		$response = $modx->runProcessor('gallery/upload', $file, array('processors_path' => $processorPath));
//echo "\nAfter run processor\n";
		//st_log($response);
		return $id;
	}
	return null;
}

/**
 * Получить цену товара или модификации по указанному прайсу
 * 
 * @param array $product_data Данные продукта
 * @param int $price_id Идентификатор прайса
 * 
 * @return float Цена продукта по выбранному прайсу
 */
function st_getProductPriceValue($product_data, $detail_data, $price_id) {
	$price_value = null;
	foreach($product_data as $key => $value) {
		if(preg_match('/^price_([0-9]+)$/', $key, $matches)) {
			$current_price_id = $key;
			if($price_id == $current_price_id) {
				$price_value = $value;
			}
			$price_default_value = $value;
		}
	}
/*	if($value) return $value;
	else return $price_default_value;

	foreach($detail_data['prices'] as $price_data) {
		if($price_data['price_id'] == $price_id && $price_data['value'] != '') {
			$price_value = $price_data['value'];
			break;
		}
		// Цена в прайсе по умолчанию
		if($price_data['is_required'] == true) {
			$price_default_value = $price_data['value'];
		}
	}
*/
	// Если цены в прайсе нет - ставим цену по умолчанию
	if($price_value === null) {
		$price_value = $price_default_value;
	}

	return $price_value;
}

/**
 * Проверка что продукт в одной из импортируемых групп
 * 
 * @param array $product Данные продукта
 * @param array $import_tags Группы для импорта
 * 
 * @return bool true если пересечения есть, false если нет
 */
function st_checkProductInTags($product, $import_tags) {
	// Если не выбрана ни одна группа считаем что теги совпадают
	if($import_tags == null || !is_array($import_tags) || count($import_tags)==0) return true;

	// Если у товара нет тегов, то считаем что он не попадает
	if(!is_array($product['tags']) || count($product['tags']) == 0) return false;

	// Проверяем что теги пересекаются
	if(count(array_intersect($product['tags'], $import_tags)) > 0) return true;
	else return false;
}
