<?php
/**
 * Функции для взаимодействия с Subtotal API
 * 
 * @package Subtotal plugin
 * @author Царёв Владимир <sau412@arikado.xyz>
 * @filesource
 */

/** Subtotal API URL const */
const SUBTOTAL_API = "https://app.subtotal.ru/";
//const SUBTOTAL_API = "https://app.dev.subtotal.ru/";

/**
 * Отправить запрос в subtotal через curl
 */
function st_send_query($type, $url, $payload = '') {
	$ch = curl_init($url);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
	if($type == "raw") {
		curl_setopt($ch, CURLOPT_HEADER, 0);
	}
	else {
		curl_setopt($ch, CURLOPT_HEADER, 1);
	}
	// Session cookie
	$session = st_get_session();
	if($session) {
		curl_setopt($ch, CURLOPT_COOKIE, "session=$session");
	}
	if($type == "put") {
		curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "PUT");
		curl_setopt($ch, CURLOPT_POSTFIELDS, $payload);
		curl_setopt($ch, CURLOPT_HTTPHEADER, ["Content-Type: application/json"]);
	}
	else if($type == "post") {
		curl_setopt($ch, CURLOPT_POST, TRUE);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $payload);
		curl_setopt($ch, CURLOPT_HTTPHEADER, ["Content-Type: application/json"]);
	}
	else {
		curl_setopt($ch, CURLOPT_POST, FALSE);
	}

	curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 60);
	curl_setopt($ch, CURLOPT_TIMEOUT, 60);
	//curl_setopt($ch, CURLOPT_HEADERFUNCTION, "st_response_header");

	$result = curl_exec($ch);
	if($type == "raw") {
		return $result;
	}

	$session = '';
	$result_data = [];
	$headers_array = [];
	$headers_flag = true;
	$reply_code = false;
	$result_split = explode("\n", $result);
	foreach($result_split as $str) {
		if($headers_flag) {
			if(!$reply_code && preg_match('/^[^ ]+ ([^ ]+)/', $str, $matches)) {
				$reply_code = $matches[1];
			}
			if(preg_match('/^Set-Cookie: .*session=([0-9a-fA-F]+);/', $str, $matches)) {
				$session = $matches[1];
			}
			if(trim($str) == '') $headers_flag = false;
			$headers_array = $str;
		}
		else {
			$result_data[] = $str;
		}
	}

	if($reply_code != 200) {
		st_log_warning([
			"message" => "Reply code is not 200 in query",
			"raw_reply" => $result,
			"function" => "st_send_query(type = $type, url = $url, payload = $payload)",
		]);
	}

	$result_str = implode("\n", $result_data);
	$data = json_decode($result_str, true, 512, JSON_INVALID_UTF8_IGNORE);

	return ["data" => $data, "session" => $session, "headers" => $headers_array];
}

/**
 * Авторизация в Subtotal
 */
function st_auth($login, $password) {
	$payload = json_encode([
		"login" => $login,
		"password" => $password,
	]);
	$response = st_send_query("post", SUBTOTAL_API.'/webapi/login', $payload);
//	var_dump($response);die();
	return $response;
}

/**
 * Получить список магазинов для данного бизнеса
 * 
 * @return array Список магазинов
 */
function st_getPoses() {
	$owner = st_get_owner();
	$url = SUBTOTAL_API.$owner.'/poslist.php?show_archive=false';
	$response = st_send_query("get", $url);
	return $response['data'];
}

/**
 * Получить список складов для данного бизнеса
 * 
 * @return array Список складов
 */
function st_getWarehouses() {
	$owner = st_get_owner();
	$url = SUBTOTAL_API.$owner.'/storelist.php?show_archive=false';
	$response = st_send_query("get", $url);
	return $response['data'];
}

/**
 * Получить каталог товаров (весь)
 * 
 * @return array Каталог товаров
 */
function st_getCatalog() {
	$owner = st_get_owner();
	$url = SUBTOTAL_API.$owner.'/webapi/goods';
	$response = st_send_query("get", $url);
	return $response['data'];
}

/**
 * Получить выгрузку товаров в указанном складе с указанным прайсом в указанных количествах
 * 
 * В отличие от getCatalog возвращает остатки и не разделяет товары и модификации
 * 
 * @param int $store Склад
 * @param int $price Прайс
 * @param int $page_num Страница (для постраничного запроса)
 * @param int $page_size Размер страницы (для постраничного запроса)
 * @param int $ch_from Дата последнего изменения
 * 
 * @return array Список товаров включая модификации и объём на указанном складе
 */
function st_getStoreGoods($store, $price, $page_num, $page_size, $ch_from = 0) {
	$owner = st_get_owner();
	$url = SUBTOTAL_API.$owner.'/data/store_goods.php?favourite_goods=0&good_type=all&page_num='.
		$page_num.'&page_size='.$page_size.'&price='.$price.
		'&return_suggestions=false&search=&store='.$store.'&tags=&totals=0&'.
		'no_store_sales=force_on&ch_from='.$ch_from;

	$response = st_send_query("get", $url);
	return $response['data'];
}

// Получить прайсы
/**
 * Получить прайсы для текущего бизнеса
 * 
 * @return array Список прайсов
 */
function st_getPrices() {
	$owner = st_get_owner();
	$url = SUBTOTAL_API.$owner.'/repricing/get_prices.php?is_service=false&show_archive=false';
	$response = st_send_query("get", $url);
	return $response['data'];
}

/**
 * Получить детальную информацию о товаре
 * 
 * Если указать id модификации, вернёт родительский товар (и модификации к нему).
 * Так же нужна ради тегов
 * 
 * @param int $good_id Идентификатор товара
 */
function st_getProductDetailed($good_id) {
	$owner = st_get_owner();
	$url = SUBTOTAL_API.$owner.'/webapi/goods/'.$good_id;
	$response = st_send_query("get", $url);
	return $response['data'];
}

/**
 * Получить группы товаров для данного бизнеса (в WooCommerce это теги)
 * 
 * @return array Список тегов
 */
function st_getTags() {
	$owner = st_get_owner();
	$url = SUBTOTAL_API.$owner.'/webapi/good_tags';
	$response = st_send_query("get", $url);
	return $response['data'];
}

/**
 * Получить картинку товара
 * 
 * Нужна потому что WordPress не умеет обращаться для загрузки картинки с пользовательскими куками
 * 
 * @param string $image_url URL изображения в subtotal
 * 
 * @return mixed содержимое изображения, двоичные данные
 */
function st_getImage($image_url) {
	$owner = st_get_owner();
	$response = st_send_query("raw", $image_url);
	return $response;
}

/**
 * Экспорт продажи в Subtotal
 * 
 * Создаёт продаже с указанными параметрами в Subtotal
 * @param array $sale Данные о продаже
 * @param integer $id Идентификатор существующей продажи
 * 
 * @return ?
 */
function st_exportSale($sale, $id = '') {
	$owner = st_get_owner();
	$url = SUBTOTAL_API.$owner.'/webapi/stock/outcomes'.($id == ''? '': "/$id");
	$payload = json_encode($sale);
	if($id == '') {
		$response = st_send_query("post", $url, $payload);
	}
	else {
		$response = st_send_query("put", $url, $payload);
	}
	st_log(["st_exportSale", $url, $sale, $payload, $id, $response]);
	return $response['data'];
}

/**
 * Получение данных продажи
 * 
 * @param integer $id Идентификатор продажи
 * 
 * @return array Продажа
 */
function st_getSale($id) {
st_log("st_getSale($id)");
	$owner = st_get_owner();
	$url = SUBTOTAL_API.$owner.'/webapi/stock/outcomes/'.$id;
	$response = st_send_query("get", $url);
	return $response['data'];
}

/**
 * Поиск среди клиентов
 * 
 * @param string $filter Строка для поиска
 * 
 * @return array Список подходящих клиентов
 */
function st_filterPartners($filter) {
	$owner = st_get_owner();
	$url = SUBTOTAL_API.$owner.'/partner/list.php?filter_text='.$filter;
//st_log(["st_filterPartners" => $url]);
	$response = st_send_query("get", $url);
	return $response['data'];
}

/**
 * Получить список бизнесов у текущего аккаунта
 * 
 * @return array Список бизнесов
 */
function st_getShops() {
	$url = SUBTOTAL_API.'subtotal_common/dktapp/shop_list';
	$response = st_send_query("get", $url);
	return $response['data'];
}

/**
 * Создание клиента (партнёра, покупателя)
 * 
 * @param array $partnerData Данные клиента
 * 
 * @return array ответ от Subtotal 
 */
function st_createPartner($partnerData) {
	$owner = st_get_owner();
	$url = SUBTOTAL_API.$owner.'/partner/editnull.php';
	$payload = json_encode($partnerData);
	$response = st_send_query("post", $url, $payload);
//var_dump("st_createPartner", $url, $partnerData, json_encode($partnerData), $payload, $response);
	return $response['data'];
}

/**
 * Отметить заказ как оплаченный
 * 
 * @param integer $order_id Номер заказа
 * @param array $payData Данные об оплате
 */
function st_payOrder($order_id, $payData) {
	$owner = st_get_owner();
	$url = SUBTOTAL_API.$owner.'/store/outcomepay'.$order_id.'.php';
	$payload = json_encode($payData);
	$response = st_send_query("post", $url, $payload);
	return $response['data'];
}

/**
 * Отладочная функция, отправляет лог на log.arikado.xyz
 * 
 * @param mixed $message Сообщение в лог (строка или массив)
 */
function st_log($message, $severity = 7) {
	$message = json_encode(["ip" => $_SERVER['REMOTE_ADDR'], "message" => $message]);
	$source = "Subtotal";
	$ch = curl_init("https://log.arikado.xyz/logger.php");

	$payload = json_encode([
		"severity" => $severity,
		"source" => "modx",
		"message" => $message,
	]);
	curl_setopt($ch, CURLOPT_POST, TRUE);
	curl_setopt($ch, CURLOPT_POSTFIELDS, $payload);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
	curl_exec($ch);

	syslog(LOG_DEBUG, "[Subtotal] ".$message);
}

function st_log_debug($message) {
	st_log($message, 7);
}

function st_log_info($message) {
	st_log($message, 6);
}

function st_log_notice($message) {
	st_log($message, 5);
}

function st_log_warning($message) {
	st_log($message, 4);
}
